﻿//-----------------------------------------------------------------------------
// Copyright 2013 by VueSoft LLC. All rights reserved.
// http://www.printablecal.com
//-----------------------------------------------------------------------------
//
// Redistribution and use in source and binary forms, with or without 
// modification, are permitted provided that the following conditions are met:
//
// 1. Redistributions of source code must retain the above copyright notice, 
//    this list of conditions and the following disclaimer.
// 
// 2. Redistributions in binary form must reproduce the above copyright notice,
//    this list of conditions and the following disclaimer in the documentation
//    and/or other materials provided with the distribution.
// 
// 3. Neither the name of "PrintableCal" nor the names of its contributors may 
//    be used to endorse or promote products derived from this software without
//    specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
// AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
// ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
// LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
// CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
// SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
// INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
// CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
// ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
// POSSIBILITY OF SUCH DAMAGE.
//
//-----------------------------------------------------------------------------
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Data.OleDb;
using System.Drawing;
using System.IO;
using System.Linq;
using System.Text;
using System.Windows.Forms;

using Vuesoft.PrintableCal.Common;
using Vuesoft.PrintableCal.Interfaces;

namespace AccessDbCalendarSource
{
   /// <summary>
   /// This dialog allows the user to add calendars from the example database.
   /// </summary>
   public partial class FormAddCalendar : Form
   {
      /// <summary>
      /// The current calendar data source collection. This collection is updated when the OK button is clicked.
      /// </summary>
      private ICalendarCollection m_calendars = null;

      /// <summary>
      /// A list of previously selected calendar identifiers associated with the Example Access Database source.
      /// </summary>
      private List<string> m_previouslySelectedCalendars = new List<string>();

      /// <summary>
      /// Construct provided the current collection of calendar data sources.
      /// </summary>
      /// <param name="calendars">The calendar data sources.</param>
      public FormAddCalendar(ICalendarCollection calendars)
      {
         InitializeComponent();
         m_calendars = calendars;
      }

      /// <summary>
      /// This function populates the list of available calendars, based on the specified database file. Any calendars that
      /// have been previously added will be automatically checked.
      /// </summary>
      private void PopulateCalendarList()
      {         
         string connectionString = @"Provider=Microsoft.ACE.OLEDB.12.0;Data Source=" + textBoxDatabase.Text;

         using (OleDbConnection connection = new OleDbConnection(connectionString))
         {
            connection.Open();

            // fill the dataset
            DataSet dataSet = new DataSet();
            
            using (OleDbDataAdapter dataAdapter = new OleDbDataAdapter("SELECT * FROM Calendars", connection))
            {
               dataAdapter.Fill(dataSet, "Calendars");

               // initialize the list
               listViewCalendars.Items.Clear();

               foreach (DataRow calendarRow in dataSet.Tables["Calendars"].Rows)
               {
                  string calendarId = calendarRow["CalendarID"].ToString();
                  string calendarName = calendarRow["CalendarName"].ToString();
                  ListViewItem item = listViewCalendars.Items.Add(calendarName, 0);
                  item.Tag = calendarId;

                  if (m_previouslySelectedCalendars.Contains(calendarId))
                  {
                     item.Checked = true;
                  }
               }
            }

            connection.Close();
         }
      }

      /// <summary>
      /// This function is commanded when the form loads. The list of previously selected calendars is initialized. The 
      /// database text box is also initialized if previously specified.
      /// </summary>
      /// <param name="sender"></param>
      /// <param name="e"></param>
      private void FormAddCalendar_Load(object sender, EventArgs e)
      {
         try
         {
            m_previouslySelectedCalendars.Clear();

            foreach (ICalendarSource calendar in m_calendars)
            {
               if (calendar.SourceName == AccessDbCalendarSource.SOURCE_NAME)
               {
                  m_previouslySelectedCalendars.Add(calendar.CalendarId);
               }
            }

            textBoxDatabase.Text = Properties.Settings.Default.DefaultDatabaseFile;
         }
         catch (Exception ex)
         {
            FormError.ShowError(ex);
         }
      }

      /// <summary>
      /// This function is commanded when the Browse button is clicked. An open file dialog is displayed. If the user selects 
      /// a file, the database text field is updated.
      /// </summary>
      /// <param name="sender"></param>
      /// <param name="e"></param>
      private void buttonBrowse_Click(object sender, EventArgs e)
      {
         try
         {
            if (openFileDialog.ShowDialog() == DialogResult.OK)
            {
               textBoxDatabase.Text = openFileDialog.FileName;
            }
         }
         catch (Exception ex)
         {
            FormError.ShowError(ex);
         }
      }

      /// <summary>
      /// This function is commanded when the database text field is changed. If the specified database file exists, the list
      /// of available calendars is refreshed.
      /// </summary>
      /// <param name="sender"></param>
      /// <param name="e"></param>
      private void textBoxDatabase_TextChanged(object sender, EventArgs e)
      {
         try
         {
            if (File.Exists(textBoxDatabase.Text))
            {
               groupBoxCalendars.Visible = true;
               PopulateCalendarList();
            }
            else
            {
               groupBoxCalendars.Visible = false;
            }
         }
         catch (Exception ex)
         {
            FormError.ShowError(ex);
         }
      }

      /// <summary>
      /// This function is commanded when the OK button is clicked. The calendar data source collection is updated to be 
      /// consistent with the calendars that are currently checked.
      /// </summary>
      /// <param name="sender"></param>
      /// <param name="e"></param>
      private void buttonOK_Click(object sender, EventArgs e)
      {
         try
         {
            // save the default database file location
            Properties.Settings.Default.DefaultDatabaseFile = textBoxDatabase.Text;
            Properties.Settings.Default.Save();

            // determine which calendars were selected previously
            List<string> staleCalendarIds = new List<string>();

            foreach (ICalendarSource source in m_calendars)
            {
               if (source.SourceName == AccessDbCalendarSource.SOURCE_NAME)
               {
                  staleCalendarIds.Add(source.CalendarId);
               }
            }

            foreach (ListViewItem item in listViewCalendars.Items)
            {
               if (item.Checked)
               {
                  string id = item.Tag.ToString();

                  if (staleCalendarIds.Contains(id))
                  {
                     staleCalendarIds.Remove(id);
                  }
                  else
                  {
                     ICalendarSource addedSource = new AccessDbCalendarSource();
                     addedSource.ItemAppearance = DrawUtils.GetRandomAppearance();
                     addedSource.CalendarId = id;
                     addedSource.CalendarName = item.Text;
                     addedSource.DataLocation = textBoxDatabase.Text;
                     m_calendars.Add(addedSource);
                  }
               }
            }

            // remove calendars that were unchecked
            for (int i = m_calendars.Count - 1; i >= 0; --i)
            {
               ICalendarSource calendarSource = m_calendars[i];

               if (staleCalendarIds.Contains(calendarSource.CalendarId))
               {
                  m_calendars.Remove(calendarSource);
               }
            }
         }
         catch (Exception ex)
         {
            FormError.ShowError(ex);
         }
      }
   }
}
